Package: plotmo
Version: 3.7.0
Title: Plot a Model's Residuals, Response, and Partial Dependence Plots
Authors@R: person(given = "Stephen",
                  family = "Milborrow",
                  role = c("aut", "cre"),
                  email = "milbo@sonic.net")
Maintainer: Stephen Milborrow <milbo@sonic.net>
Depends: R (>= 3.4.0), Formula (>= 1.2-3), plotrix
Description: Plot model surfaces for a wide variety of models
        using partial dependence plots and other techniques.
        Also plot model residuals and other information on the model.
Suggests: C50 (>= 0.1.0-24), earth (>= 5.1.2), gbm (>= 2.1.1), glmnet
        (>= 2.0.5), glmnetUtils (>= 1.0.3), MASS (>= 7.3-51), mlr (>=
        2.12.1), neuralnet (>= 1.33), partykit (>= 1.2-2), pre (>=
        0.5.0), rpart (>= 4.1-15), rpart.plot (>= 3.0.8)
License: GPL-3
URL: http://www.milbo.users.sonic.net
NeedsCompilation: no
Packaged: 2026-01-04 01:38:29 UTC; milbo
Author: Stephen Milborrow [aut, cre]
Repository: CRAN
Date/Publication: 2026-01-09 09:30:17 UTC
Built: R 4.6.0; ; 2026-04-25 16:56:34 UTC; unix
