importFrom(plotrix, thigmophobe.labels)
importFrom(Formula, Formula)
export(plotmo)
export(plotres)
export(plot_gbm)
export(plot_glmnet)
# by convention, the prefix "plotmo_" is for
# standard functions and "plotmo." is for methods
# (but check.index is a historical name)
export(check.index)
export(plotmo.convert.na.nresponse)
export(plotmo.pairs)
export(plotmo.pint)
export(plotmo.predict)
export(plotmo.prolog)
export(plotmo.residtype)
export(plotmo.singles)
export(plotmo.type)
export(plotmo.x)
export(plotmo.y)
export(plotmo.y.default)
export(plotmo_cum)
export(plotmo_fitted)
export(plotmo_nresponse)
export(plotmo_predict)
export(plotmo_prolog)
export(plotmo_resplevs)
export(plotmo_response)
export(plotmo_rinfo)
export(plotmo_rsq)
export(plotmo_standardizescale)
export(plotmo_type)
export(plotmo_y)
S3method(plotmo.convert.na.nresponse, default)
S3method(plotmo.convert.na.nresponse, rq)
S3method(plotmo.convert.na.nresponse, rqs)
S3method(plotmo.pairs, C5.0)
S3method(plotmo.pairs, default)
S3method(plotmo.pairs, gbm)
S3method(plotmo.pairs, GBMFit)
S3method(plotmo.pairs, gpe)
S3method(plotmo.pairs, parties)
S3method(plotmo.pairs, party_plotmo)
S3method(plotmo.pairs, pre)
S3method(plotmo.pairs, randomForest)
S3method(plotmo.pairs, rpart)
S3method(plotmo.pairs, train)
S3method(plotmo.pairs, WrappedModel)
S3method(plotmo.pint, default)
S3method(plotmo.pint, earth)
S3method(plotmo.pint, Gam)
S3method(plotmo.pint, gam)
S3method(plotmo.pint, glm)
S3method(plotmo.pint, lm)
S3method(plotmo.pint, quantregForest)
S3method(plotmo.pint, rq)
S3method(plotmo.pint, rqs)
S3method(plotmo.predict, bagging)
S3method(plotmo.predict, biglm)
S3method(plotmo.predict, boosting)
S3method(plotmo.predict, bruto)
S3method(plotmo.predict, clm)
S3method(plotmo.predict, cosso)
S3method(plotmo.predict, cv.glmnet)
S3method(plotmo.predict, default)
S3method(plotmo.predict, defaultm)
S3method(plotmo.predict, gbm)
S3method(plotmo.predict, GBMFit)
S3method(plotmo.predict, glmnet)
S3method(plotmo.predict, glmnet.formula)
S3method(plotmo.predict, lars)
S3method(plotmo.predict, lda)
S3method(plotmo.predict, list)
S3method(plotmo.predict, mvr)
S3method(plotmo.predict, nn)
S3method(plotmo.predict, party_plotmo)
S3method(plotmo.predict, qda)
S3method(plotmo.predict, quantregForest)
S3method(plotmo.predict, rpart)
S3method(plotmo.predict, rq)
S3method(plotmo.predict, rqs)
S3method(plotmo.predict, svm)
S3method(plotmo.predict, WrappedModel)
S3method(plotmo.prolog, C5.0)
S3method(plotmo.prolog, cv.glmnet)
S3method(plotmo.prolog, default)
S3method(plotmo.prolog, gbm)
S3method(plotmo.prolog, GBMFit)
S3method(plotmo.prolog, glmnet)
S3method(plotmo.prolog, model_fit)
S3method(plotmo.prolog, parties)
S3method(plotmo.prolog, party)
S3method(plotmo.prolog, pre)
S3method(plotmo.prolog, train)
S3method(plotmo.prolog, WrappedModel)
S3method(plotmo.prolog, xgb.Booster)
S3method(plotmo.residtype, default)
S3method(plotmo.residtype, rpart)
S3method(plotmo.residtype, train)
S3method(plotmo.singles, C5.0)
S3method(plotmo.singles, cv.glmnet)
S3method(plotmo.singles, default)
S3method(plotmo.singles, gbm)
S3method(plotmo.singles, GBMFit)
S3method(plotmo.singles, glmnet)
S3method(plotmo.singles, parties)
S3method(plotmo.singles, party_plotmo)
S3method(plotmo.singles, pre)
S3method(plotmo.singles, randomForest)
S3method(plotmo.singles, rpart)
S3method(plotmo.singles, train)
S3method(plotmo.singles, WrappedModel)
S3method(plotmo.type, bruto)
S3method(plotmo.type, clm)
S3method(plotmo.type, cosso)
S3method(plotmo.type, default)
S3method(plotmo.type, fda)
S3method(plotmo.type, knn3)
S3method(plotmo.type, lars)
S3method(plotmo.type, lda)
S3method(plotmo.type, nnet)
S3method(plotmo.type, qda)
S3method(plotmo.type, rpart)
S3method(plotmo.type, train)
S3method(plotmo.type, tree)
S3method(plotmo.type, varmod)
S3method(plotmo.x, default)
S3method(plotmo.x, gbm)
S3method(plotmo.x, GBMFit)
S3method(plotmo.x, mars)
S3method(plotmo.x, varmod)
S3method(plotmo.y, default)
S3method(plotmo.y, gbm)
S3method(plotmo.y, GBMFit)
S3method(plotmo.y, lognet)
S3method(plotmo.y, mrelnet)
S3method(plotmo.y, multnet)
S3method(plotmo.y, varmod)
importFrom("grDevices", "as.graphicsAnnot", "col2rgb", "gray", "xy.coords")
importFrom("graphics", "abline", "axis", "box", "grid", "image",
     "legend", "lines", "mtext", "par", "plot", "points",
     "polygon", "rect", "segments", "strheight", "strwidth",
     "text", "xinch", "yinch")
importFrom("stats", "approx", "coef", "cor.test", "density",
     "deviance", "df.residual", "family", "formula", "getCall", "hatvalues", "loess",
     "lowess", "median", "model.matrix", "na.omit", "na.pass",
     "pnorm", "predict", "qqline", "qqnorm", "quantile", "qt", "rnorm",
     "sd", "update", "weighted.mean")
importFrom("utils", "assignInMyNamespace", "head", "str")
